@@ -16,7 +16,7 @@
  *   See the GNU General Public License for more details.                 *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
- *   along with this program.  If not, see http://www.gnu.org/licenses/.  *
+ *   along with this program.  If not, see https://gnu.org/licenses/.     *
  *                                                                        *
  **************************************************************************/
 
@@ -563,7 +563,7 @@ void show_help(void)
 	bottombars(oldmenu);
 
 #ifdef ENABLE_BROWSER
-	if (oldmenu & (MBROWSER|MWHEREISFILE|MGOTODIR))
+	if (oldmenu & (MBROWSER|MGOTODIR|MWHEREISFILE))
 		browser_refresh();
 	else
 #endif
